function [mps_, gamma_] = CANONICAL(mps__, gamma__, c0_, c1_, thresh_, nkeep_)
  %% fix a mps into a canonical form from site: c0_ to c1_
  %% if c0_<c1_, we have left canocial: M[c0] G A[c0+1] ... A[c1]
  %% if c0_>c1_, we have right canocial: M[c1] G B[c1+1] ... B[c0]
  %%
  %% mps_: an array of rank 3 tensors
  %% [1] --- [3]
  %%      |
  %%     [2]
  %% gamma: a matrix   [1]-[2]
 
  mps_ = mps__;
  gamma_ = gamma__;
  nsite = length(mps_);

  dd = c1_-c0_;
  if (dd>0) %% left canonicalize
    for isite = c0_+1 :1: c1_
      tmp = mps_(isite);
      if ~isempty(gamma_)
        tmp = contract(gamma_, [2], tmp, [1]);
      end
      [mps_(isite), gamma_, vt] = QSVD(tmp, [1,2], thresh_, nkeep_);

      if (isite < nsite)
        mps_(isite+1) = contract(vt, [2], mps_(isite+1), [1]);
      end
    end

  elseif (dd<0) %% right canonicalize
    for isite = c0_ :-1: c1_+1
      tmp = mps_(isite);
      if ~isempty(gamma_)
        tmp = contract(tmp, [3], gamma_, [1]);
      end
      [u, gamma_, mps_(isite)] = QSVD(tmp, [1], thresh_, nkeep_);
      
      if (isite > 1)
        mps_(isite-1) = contract(mps_(isite-1), [3], u, [1]);
      end
    end
  end
end
