function [mps_, energy_last] = DMRG_1site(mps_, effL_, effR_, mpo_, dkeep_, econv_, nsweep_)
  nsite = length(mps_);
  %% generate effective Hamiltonian
  %%      ---[3]          -----mps[i]-
  %%      |               |        |
  %% arrL[i]-[2]  =  arrL[i-1]-mpo[i]-
  %%      |               |        |
  %%      ---[1]          ----mps*[i]-
  %%
  arrR = [effR_];
  arrL = [effL_];
  for isite = 1 : nsite-1
    effL = contract(arrL(end), [1], conj(mps_(isite)), [1]);
    effL = contract(effL, [1,3], mpo_(isite), [1,2]);
    effL = contract(effL, [1,3], mps_(isite), [1,2]);
    arrL = [arrL, effL];
  end

  %% 
  energy = 0;
  energy_last = 0;
  conv = false;
  tt = tic;
  for sweep = 1 : nsweep_ 
    %% from right to left
    for isite = nsite : -1 : 2
      ts = tic;
      tag1 = mps_(isite-1).info.itags;
      tag2 = mps_(isite).info.itags;
      %% update MPS[isite]
      [wfn, energy, chk, err] = EIGS([arrL(end), mpo_(isite), arrR(end)], mps_(isite), [], [], []);
      if any(~chk)
        message = sprintf('%1.2e   ', err);
        fprintf('WARNING: EIGS not converged: [%s]\n', message)
      end
      
      %% shift center left to MPS[isite-1]
      [u, s, vt] = QSVD(wfn, [1], 1E-16, dkeep_);
      us = contract(u, [2], s, [1]);
      mps_(isite-1) = contract(mps_(isite-1), [3], us, [1]);
      mps_(isite) = vt;
      mps_(isite-1).info.itags = tag1;
      mps_(isite).info.itags = tag2;

      %% update effective Hamiltonian 
      arrL = arrL(1:end-1);
      effR = contract(conj(mps_(isite)), [3], arrR(end), [1]);
      effR = contract(effR, [2,3], mpo_(isite), [2,4]);
      effR = contract(effR, [4,2], mps_(isite), [2,3]);
      arrR = [arrR, effR];
      fprintf('*** SWEEP: %3d, R2L @ %3u; E = %23.16f >>> %10.2f sec \n', sweep, isite, energy, toc(ts));
    end
    
    % from left to right
    for isite = 1 : nsite-1
      ts = tic;
      tag1 = mps_(isite).info.itags;
      tag2 = mps_(isite+1).info.itags;
      %% update MPS[isite]
      [wfn, energy, chk, err] = EIGS([arrL(end),  mpo_(isite), arrR(end)], mps_(isite), [], [], []);
      if any(~chk)
        message = sprintf('%1.2e   ', err);
        fprintf('WARNING: EIGS not converged: [%s]\n', message)
      end

       %% shift center right to MPS[isite+1]
      [u, s, vt] = QSVD(wfn, [1,2], 1E-16, dkeep_);
      mps_(isite) = u;
      svt = contract(s, [2], vt, [1]);
      mps_(isite+1) = contract(svt, [2], mps_(isite+1), [1]);
      mps_(isite).info.itags = tag1;
      mps_(isite+1).info.itags = tag2;

      %% update effective Hamiltonian
      arrR = arrR(1:end-1);
      effL = contract(arrL(end), [1], conj(mps_(isite)), [1]);
      effL = contract(effL, [1,3], mpo_(isite), [1,2]);
      effL = contract(effL, [1,3], mps_(isite), [1,2]);
      arrL = [arrL, effL];
      fprintf('*** SWEEP: %3d, L2R @ %3u; E = %23.16f >>> %10.2f sec \n', sweep, isite, energy, toc(ts));
    end

    if (abs(energy_last/energy - 1) < econv_)
      energy_last = energy;
      conv = true;
      break;
    end
    energy_last = energy;
  end

  if (conv)
    fprintf('****** DMRG CONVERGED ******                                       %10.2f sec \n', toc(tt));
  else
    fprintf('****** DMRG not CONVERGED ******                                   %10.2f sec \n', toc(tt));
  end
end
