function [out_] = QDSUM(varargin)
  nsize = length(varargin) - 1; 
  idx = varargin{end};
  out_ =  DSUM(varargin{1}, varargin{2}, idx);
  for ii = 3:nsize
    out_ = DSUM(out_, varargin{ii}, idx);
  end
end

function [out_] = DSUM(inpA_, inpB_, idx_)
%% ref: https://stackoverflow.com/questions/52599432/matlab-using-two-index-arrays-to-operate-on-a-sub-matrix
  if isempty(inpA_) 
    out_ = inpB_; return; 
  end
  if isempty(inpB_)
    out_ = inpA_; return; 
  end

  tA = inpA_;
  rankA = length(tA.Q);
  tB = inpB_;
  rankB = length(tB.Q);
  assert((rankA == rankB),'Error in QDSUM');

  nvar = length(idx_);
  dimA = cell(nvar,1);
  iA = cell(nvar,1);
  jA = cell(nvar,1);
  dimB = cell(nvar,1);
  iB = cell(nvar,1);
  jB = cell(nvar,1); 

  for nn = 1 : nvar
    idx = idx_(nn);
    [qA, dimA{nn}] = getQDimQS(tA, idx); 
    [~, iA{nn}] = matchIndex(tA.Q{idx}, qA, '-s');

    [qB, dimB{nn}] = getQDimQS(tB, idx); 
    [~, iB{nn}] = matchIndex(tB.Q{idx}, qB, '-s');

    [aa, bb] = matchIndex(qA,qB);
    jB{nn} = zeros(1,size(qA,1)); 
    jB{nn}(aa) = bb;
    jA{nn} = zeros(1,size(qB,1)); 
    jA{nn}(bb) = aa;
  end

  for ii = 1:length(tA.data)
    jj = [];
    for nn = 1 : nvar
      jj = [jj; jB{nn}(iA{nn}(ii))];
    end
    if sum(jj) ~= 0
      startInd = ones(1,rankA);
      endInd = size(tA.data{ii}); 
      endInd(end+1:rankA) = 1;
      for nn = 1 : nvar
        idx = idx_(nn);
        startInd(idx) = dimA{nn}(iA{nn}(ii))+1;
        if jj(nn)~=0
          endInd(idx) = endInd(idx) + dimB{nn}(jj(nn));
        end
      end

      ndims = cell(rankA, 1);
      for kk = 1 : rankA
        ndims{kk} = startInd(kk) : endInd(kk);
      end
      S.type = '()';
      S.subs = ndims;
      tA.data{ii} = subsasgn(tA.data{ii}, S, 0);
    end
  end

  for ii = 1:length(tB.data)
    jj = []; 
    for nn = 1 : nvar
      jj = [jj; jA{nn}(iB{nn}(ii))];
    end
    if sum(jj)~=0
      startInd = ones(1,rankB);
      endInd = size(tB.data{ii});
      endInd(end+1:rankB) = 1;
      for nn = 1 : nvar
        idx = idx_(nn);
        if jj(nn)~=0
          startInd(idx) = dimA{nn}(jj(nn))+1;
          endInd(idx) = endInd(idx) + dimA{nn}(jj(nn));
        end
      end

      ndims = cell(rankB, 1);
      for kk = 1 : rankB
        ndims{kk} = startInd(kk) : endInd(kk);
      end
      S.type = '()';
      S.subs = ndims;
      tB.data{ii} = subsasgn(zeros(endInd), S, tB.data{ii});
    end
  end
  tmpA = tA;
  for idata = 1 : length(tmpA.data)
    tmpA.data{idata} = ones(size(tmpA.data{idata}));
  end
  tmpB = tB;
  for idata = 1 : length(tmpB.data)
    tmpB.data{idata} = ones(size(tmpB.data{idata}));
  end
  out_ = realmin*tmpA + realmin*tmpB + tA + tB;
end
