function [outT_, uniT_] = QFUSE(inpT_, idx_, flag_, varargin)
  %% inpT_: QSpace tensor
  %% idx_: array of tensor indices to be fused
  %% flag_: 'I': new tensor index is ingoing and put at the first; 
  %%        'O': new tensor index is outgoing and put at the end;
  %% varargin{1}: a character array for new tensor index's itag 

  %% construct unitary tensor uniT_[T(idx_(1)),...T(idx_(end)), new_index*]
  
  if ~(isa(inpT_, 'QSpace'))
    inpT_ = QSpace(inpT_);
  end
  
  ll = length(idx_);
  uniT_ = QSpace;
  % first index
  if (length(inpT_.info.itags{idx_(1)}) ~= 0) && (inpT_.info.itags{idx_(1)}(end) == '*') %out
    uniT_ = getIdentity(inpT_, [idx_(1)], '-0');
    uniT_ = conj(uniT_);
  else
    uniT_ = getIdentity(inpT_, [idx_(1)]);
  end;

  % the rest
  if (ll > 1)
    for ii = 2:ll
      kk = idx_(ii);
      if (length(inpT_.info.itags{kk}) ~= 0) && (inpT_.info.itags{kk}(end) == '*') %out
        tmp = conj(getIdentity(inpT_, [kk], '-0'));
        tmpI = getIdentity(tmp, [2], uniT_, [ii]);
        tmpI = contract(tmp, [2], tmpI, [1]);
        uniT_ = contract(uniT_, [ii], tmpI, [2]);
      else
        tmpI = getIdentity(inpT_, [kk], uniT_, [ii]);
        uniT_ = contract(uniT_, [ii], tmpI, [2]);
      end;
    end;
  end;

  % fuse 
  if (length(varargin) ~= 0)
    uniT_.info.itags{end} = [varargin{1}, '*'];
  end

  outT_ = QSpace;
  switch flag_(1)
  case 'I'
    outT_ = contract(conj(uniT_), [1:ll], inpT_, idx_);
    return;
  case 'O'
    uniT_ = contract(uniT_, [ll+1], getIdentity(uniT_, [ll+1], '-0'), [1]);
    outT_ = contract(inpT_, idx_, conj(uniT_), [1:ll]);
    return;
  otherwise
    error('QFUSE failed. Ivalid input parameter flag_ %s.',flag_);
  end 
end
